import logging

from sqlalchemy import select
from sqlalchemy.orm import aliased
from sqlalchemy.orm.exc import MultipleResultsFound

from schema import create_session, Author
from schema.almamodel import BmmvObsproject, BmmvObsproposal, t_mv_obsproposal, \
    t_asp_obs_proposal_author, BmmvSchedblock, ShiftlogEntry, AquaExecblock, AsaProductFile, NgasFiles, \
    AsaScience, AsaEnergy

LOG = logging.getLogger('ingestion')

session = create_session('ALMA')


def getproject(projectid):
    project = session.query(BmmvObsproject).filter_by(prj_archive_uid = projectid).first()
    return project

def getabstract(projectid):
    proposal = session.query(BmmvObsproposal).filter_by(obsproject_archive_uid=projectid).first()
    return proposal.abstract_text

def getauthors(projectid):
    """
    Return project authors
    :param projectid: project for which authors are desired
    :return: list of authors info
    """

    """
    select aut.firstname, aut.lastname, aut.account_id, aut.author_index
    from ALMA.ASP_OBS_PROPOSAL_AUTHOR aut
    inner join ALMA.MV_OBSPROPOSAL mop
    on mop.archive_uid = aut.proposal_archive_uid
    where mop.obsproject_archive_uid = ?
    ;
    """

    s = select([t_asp_obs_proposal_author.c.firstname, t_asp_obs_proposal_author.c.lastname, t_asp_obs_proposal_author.c.account_id, t_asp_obs_proposal_author.c.author_index]) \
        .select_from(
        t_asp_obs_proposal_author.join(t_mv_obsproposal, t_mv_obsproposal.c.archive_uid==t_asp_obs_proposal_author.c.proposal_archive_uid)) \
        .where(t_mv_obsproposal.c.obsproject_archive_uid==projectid)

    result = [
        Author(author_id=None, firstname=r[0], lastname=r[1], username=r[2], project_code=projectid, is_pi=r[3]==0)
        for r in session.execute(s).fetchall()
    ]

    return result


def getalmaoushierarchy(execblock):
    """
    Build the ALMA OUS hierarchy for this execblock and return the MOUS, GOUS, and SOUS ids
    :param execblock:
    :return: MOUS, GOUS, SOUS ids
    """

    """
    select SGOUS_STATUS_UID, sbs.GOUS_STATUS_UID, sbs.MOUS_STATUS_UID
    FROM BMMV_SCHEDBLOCK sbs
    join ALMA.SHIFTLOG_ENTRIES shifts ON shifts.SE_SB_ID = sbs.ARCHIVE_UID
    join AQUA_EXECBLOCK ebs ON ebs.EXECBLOCKUID = shifts.SE_EB_UID
    where ebs.EXECBLOCKUID= ?
;
    """

    sbs = aliased(BmmvSchedblock)
    shifts = aliased(ShiftlogEntry)
    ebs = aliased(AquaExecblock)

    execblockuid = execblock.replace('___', '://').replace('_', '/')
    LOG.info(f"Execblock UID: {execblockuid}")

    sb, sh, eb = session.query(sbs, shifts, ebs).\
        filter(shifts.se_sb_id==sbs.archive_uid).\
        filter(ebs.execblockuid==shifts.se_eb_uid).\
        filter(ebs.execblockuid==execblockuid).\
        one()

    LOG.info(f"Execblock: {execblock} MOUS: {sb.mous_status_uid} GOUS: {sb.gous_status_uid} SOUS: {sb.sgous_status_uid}")
    return sb.mous_status_uid, sb.gous_status_uid, sb.sgous_status_uid


def getalmacalfiles(mous):
    """
    :param mous:
    :return: list of calibration products
    """

    """
    select FILE_ID,FILE_SIZE from ngas.ngas_files where file_id in (select ngas_file_id from
    alma.asa_product_files where ASA_OUS_ID=? and FILE_CLASS in ('calibration','script'));
    """

    LOG.info(f"Getting calibration products for mous {mous}")

    files = session.query(AsaProductFile.ngas_file_id, NgasFiles.file_size). \
        filter(AsaProductFile.asa_ous_id==mous). \
        filter(AsaProductFile.file_class.in_(['calibration','script'])). \
        filter(AsaProductFile.ngas_file_id==NgasFiles.file_id) . \
        all()

    for file in files:
        LOG.info(f"File name: {file[0]} size: {file[1]}")

    return [(file[0], file[1]) for file in files]


def getschedblockname(mous):
    """
    Return the sched block name given the mous id

    :param mous: MOUS uid
    :return: schedblock name
    """

    """
    SELECT SB_NAME FROM BMMV_SCHEDBLOCK WHERE STATUS NOT IN ('Suspended', 'Cancelled') AND STATUS NOT LIKE 'CSV%'
    AND MOUS_STATUS_UID='uid://A001/X1284/X15db';
    
    """

    LOG.info(f"Mous id for sched block name: {mous}")
    sbs = aliased(BmmvSchedblock)
    try:
        sb = session.query(sbs). \
            filter(sbs.status.notin_(['Suspended', 'Cancelled'])). \
            filter(~(sbs.status.ilike('csv%'))). \
            filter(sbs.mous_status_uid==mous). \
            one_or_none()
    except MultipleResultsFound:
        LOG.warning("Multiple results found for SB name of {}.  Setting to NULL.".format(mous))
        return None

    if sb is not None:
        LOG.info(sb.sb_name)
        return sb.sb_name
    else:
        LOG.info("No Appropriate SB Found.")
        return None


def get_source_list(mous):
    """
    Get source list from MOUS id
    :param mous:
    :return:
    """
    # mous = 'uid://A001/X1284/X15db'

    # _SOURCE_DATA_QUERY = """SELECT SOURCE_NAME, RA_SOURCE, DEC_SOURCE, INT_TIME, SPATIAL_SCALE_MIN, SPATIAL_SCALE_MAX, DATASET_ID FROM ALMA.ASA_SCIENCE WHERE ASA_OUS_ID=:mous_uid AND DATASET_ID LIKE :like_this"""

    source_list = session.query(AsaScience). \
        filter(AsaScience.asa_ous_id==mous). \
        filter(AsaScience.dataset_id.ilike(mous + '%')). \
        all()

    return source_list


def get_spectral_window_data(source):
    # _SPW_DATA_FOR_SOURCE = """SELECT SPW_NUM, FREQUENCY_MIN, FREQUENCY_MAX, BANDWIDTH, CHANNEL_NUM, RESOLUTION_MIN, RESOLUTION_MAX FROM ALMA.ASA_ENERGY WHERE ASA_DATASET_ID=:dsid"""
    spectral_window_data = session.query(AsaEnergy). \
        filter(AsaEnergy.asa_dataset_id==source.dataset_id). \
        all()

    return spectral_window_data


def update_alma_source_data():
    pass


def update_alma_spectral_window_data():
    pass


def update_alma_spectral_window_source_data():
    pass

